/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMApplicationCache.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMApplicationCache.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/PrivateGdomDOMApplicationCache.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateIntGdomDOMApplicationCache.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::DOMApplicationCache* obj)
{
    return WebCore::cacheDOMObject<WebCore::DOMApplicationCache, GDOMDOMApplicationCache>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMDOMApplicationCache::GDOMDOMApplicationCache(PassRefPtr<WebCore::DOMApplicationCache> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMDOMApplicationCache::~GDOMDOMApplicationCache()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

void
gdom_dom_application_cache_update (GdomDOMApplicationCache *thiz)
{
    g_return_if_fail (thiz);
    WebCore::DOMApplicationCache * item = (gdom_dom_application_cache_get_private(thiz));
    g_return_if_fail (item);
    WebCore::ExceptionCode ec = 0;
    item->update(ec);

}

void
gdom_dom_application_cache_swap_cache (GdomDOMApplicationCache *thiz)
{
    g_return_if_fail (thiz);
    WebCore::DOMApplicationCache * item = (gdom_dom_application_cache_get_private(thiz));
    g_return_if_fail (item);
    WebCore::ExceptionCode ec = 0;
    item->swapCache(ec);

}

gchar *
gdom_dom_application_cache_item (GdomDOMApplicationCache *thiz, gulong index)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::DOMApplicationCache * item = (gdom_dom_application_cache_get_private(thiz));
    g_return_val_if_fail (item, 0);
    WebCore::ExceptionCode ec = 0;
    gchar * res = _G_GET_UTF8(item->item(index, ec));
    return res;

}


/* XXX TODO: custom function gdom_dom_application_cache_add */


/* XXX TODO: custom function gdom_dom_application_cache_remove */


/* XXX TODO: custom function gdom_dom_application_cache_add_event_listener */


/* XXX TODO: custom function gdom_dom_application_cache_remove_event_listener */


/* XXX TODO: custom function gdom_dom_application_cache_dispatch_event */

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "DOMApplicationCache.h"
#include <gdom/GdomDOMApplicationCache.h>
#include <gdom/PrivateGdomDOMApplicationCache.h>
#include <gdom/PrivateIntGdomDOMApplicationCache.h>
#include <gdom/GdomDOMApplicationCacheprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomDOMApplicationCache, gdom_dom_application_cache, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomDOMApplicationCachePrivate {
    class WebCore::DOMApplicationCache *item;
};

#define GDOM_DOM_APPLICATION_CACHE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_DOM_APPLICATION_CACHE, GdomDOMApplicationCachePrivate))
*/
G_END_DECLS
WebCore::DOMApplicationCache *
gdom_dom_application_cache_get_private(GdomDOMApplicationCache *request)
{
    GDOMDOMApplicationCache *res;
    res = static_cast<GDOMDOMApplicationCache*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::DOMApplicationCache*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_STATUS,
    PROP_LENGTH,
    PROP_ONCHECKING,
    PROP_ONERROR,
    PROP_ONNOUPDATE,
    PROP_ONDOWNLOADING,
    PROP_ONPROGRESS,
    PROP_ONUPDATEREADY,
    PROP_ONCACHED,
};


static void gdom_dom_application_cache_finalize(GObject* object)
{
    /*GdomDOMApplicationCache* request = GDOM_DOM_APPLICATION_CACHE(object);*/
    /*GdomDOMApplicationCachePrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_dom_application_cache_parent_class)->finalize(object);
}

static void gdom_dom_application_cache__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomDOMApplicationCache *self = GDOM_DOM_APPLICATION_CACHE(object);
    WebCore::DOMApplicationCache * gself = (gdom_dom_application_cache_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_dom_application_cache__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomDOMApplicationCache *self = GDOM_DOM_APPLICATION_CACHE(object);
    WebCore::DOMApplicationCache * gself = (gdom_dom_application_cache_get_private(self));
    switch (prop_id) {
    case PROP_STATUS:
    {
         g_value_set_uint (value, gself->status());
         break;
    }
    case PROP_LENGTH:
    {
         g_value_set_ulong (value, gself->length());
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_dom_application_cache_class_init(GdomDOMApplicationCacheClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_dom_application_cache_finalize;
    g_klass->set_property = gdom_dom_application_cache__set_property_fn;
    g_klass->get_property = gdom_dom_application_cache__get_property_fn;

    g_object_class_install_property (g_klass, PROP_STATUS,
            g_param_spec_uint ("status", /* name */
                "dom_application_cache_status", /* short description */
                "read-only  gushort DOMApplicationCache.status", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXUINT16, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_LENGTH,
            g_param_spec_ulong ("length", /* name */
                "dom_application_cache_length", /* short description */
                "read-only  gulong DOMApplicationCache.length", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomDOMApplicationCachePrivate));*/
}

static void gdom_dom_application_cache_init(GdomDOMApplicationCache* request)
{
    /*GdomDOMApplicationCachePrivate* priv = GDOM_DOM_APPLICATION_CACHE_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

